# -*- coding: utf-8 -*-

"""
todo: doc string
"""

import typing as T
import dataclasses

from acore_constants.api import ServerLifeCycle

if T.TYPE_CHECKING:  # pragma: no cover
    from .main import Env


@dataclasses.dataclass
class Server:
    """
    Per Game Server configuration.

    :param id: Server id, the naming convention is ``${env_name}-${server_name}``.
    :param ec2_ami_id: the AMI id for the game server.
    :param ec2_instance_type: the EC2 instance type for the game server.
    :param ec2_subnet_id: the EC2 subnet id for the game server.
    :param ec2_key_name: the EC2 ssh key name for the game server.
    :param ec2_eip_allocation_id: if you need a static IP, then create
        an Elastic IP address and put the allocation id here. otherwise,
        use the automatic public IP address.
    :param acore_soap_app_version: the acore_soap_app-project git tag for bootstrap.
    :param acore_server_bootstrap_version: the acore_server_bootstrap-project
        git tag for bootstrap.
    :param db_snapshot_id: the snapshot id to create the RDS DB instance.
    :param db_instance_class: the RDS instance class for the game database.
    :param db_admin_password: the RDS admin password, we need this password.
        to create the database user for game server.
    :param db_username: the database user for game server.
    :param db_password: the database password for game server.
    :param lifecycle: the logic "game server (both EC2 and RDS)" lifecycle definition.
    :param authserver_conf: custom config for authserver.conf.
    :param worldserver_conf: custom config for worldserver.conf.
    :param mod_lua_engine_conf: custom config for mod_LuaEngine.conf.
    """

    id: T.Optional[str] = dataclasses.field(default=None)
    # EC2 related
    ec2_ami_id: T.Optional[str] = dataclasses.field(default=None)
    ec2_instance_type: T.Optional[str] = dataclasses.field(default=None)
    ec2_subnet_id: T.Optional[str] = dataclasses.field(default=None)
    ec2_key_name: T.Optional[str] = dataclasses.field(default=None)
    ec2_eip_allocation_id: T.Optional[str] = dataclasses.field(default=None)
    acore_soap_app_version: T.Optional[str] = dataclasses.field(default=None)
    acore_server_bootstrap_version: T.Optional[str] = dataclasses.field(default=None)
    # RDS related
    db_snapshot_id: T.Optional[str] = dataclasses.field(default=None)
    db_instance_class: T.Optional[str] = dataclasses.field(default=None)
    db_admin_password: T.Optional[str] = dataclasses.field(default=None)
    db_username: T.Optional[str] = dataclasses.field(default=None)
    db_password: T.Optional[str] = dataclasses.field(default=None)
    # EC2 and RDS related
    lifecycle: T.Optional[str] = dataclasses.field(default=None)
    # authserver.conf, worldserver.conf, ...
    authserver_conf: T.Dict[str, str] = dataclasses.field(default_factory=dict)
    worldserver_conf: T.Dict[str, str] = dataclasses.field(default_factory=dict)
    mod_lua_engine_conf: T.Dict[str, str] = dataclasses.field(default_factory=dict)

    def __post_init__(self):
        if self.lifecycle not in [
            ServerLifeCycle.running,
            ServerLifeCycle.smart_running,
            ServerLifeCycle.stopped,
            ServerLifeCycle.deleted,
        ]:  # pragma: no cover
            raise ValueError(f"{self.lifecycle!r} is not a valid lifecycle definition!")


@dataclasses.dataclass
class ServerMixin:
    servers: T.Dict[str, Server] = dataclasses.field(default_factory=dict)

    @property
    def server_blue(self) -> Server:
        return self.servers["blue"]

    @property
    def server_green(self) -> Server:
        return self.servers["green"]



# -*- coding: utf-8 -*-

from pathlib import Path

dir_here = Path(__file__).absolute().parent
PACKAGE_NAME = dir_here.name

dir_project_root = dir_here.parent

# ------------------------------------------------------------------------------
# Virtual Environment Related
# ------------------------------------------------------------------------------
dir_venv = dir_project_root / ".venv"
dir_venv_bin = dir_venv / "bin"

# virtualenv executable paths
bin_pytest = dir_venv_bin / "pytest"

# test related
dir_htmlcov = dir_project_root / "htmlcov"
path_cov_index_html = dir_htmlcov / "index.html"
dir_unit_test = dir_project_root / "tests"


# ------------------------------------------------------------------------------
# Config Management Related
# ------------------------------------------------------------------------------
dir_home = Path.home()  # ${HOME}
dir_home_project_root = dir_home / ".projects" / PACKAGE_NAME

dir_config = dir_project_root / "config"
# where you store the non-sensitive config data
path_config_json = dir_config / "config.json"
# where you store the sensitive config dat
path_config_secret_json = dir_home_project_root / "config-secret.json"



# -*- coding: utf-8 -*-

"""
注, 这个模块不属于公开 API 的一部分, 仅仅用于在 acore_server_config 项目本身内部用于部署
配置数据. 如果你在其他项目中引用了这个项目, 请使用 acore_server_config.api 模块中的内容.
"""

import json

from ..paths import path_config_json, path_config_secret_json
from ..runtime import IS_LOCAL

from .define import EnvEnum, Env, Config

if IS_LOCAL:
    # ensure that the config-secret.json file exists
    # it should be at the ${HOME}/.projects/wserver_infra/config-secret.json
    # this code block is only used to onboard first time user of this
    # project template. Once you know about how to handle the config-secret.json file,
    # you can delete this code block.
    if not path_config_secret_json.exists():  # pragma: no cover
        path_config_secret_json.parent.mkdir(parents=True, exist_ok=True)
        path_config_secret_json.write_text(
            json.dumps(
                {
                    "_shared": {},
                    EnvEnum.sbx.value: {"password": f"{EnvEnum.sbx.value}.password"},
                    EnvEnum.tst.value: {"password": f"{EnvEnum.tst.value}.password"},
                    EnvEnum.prd.value: {"password": f"{EnvEnum.prd.value}.password"},
                },
                indent=4,
            )
        )

    # read non-sensitive config and sensitive config from local file system
    config = Config.read(
        env_class=Env,
        env_enum_class=EnvEnum,
        path_config=path_config_json,
        path_secret_config=path_config_secret_json,
    )
else:
    raise NotImplementedError



# -*- coding: utf-8 -*-

from acore_server_config import api


def test():
    _ = api

    _ = api.IS_LOCAL
    _ = api.IS_GITHUB_CI
    _ = api.IS_EC2
    _ = api.IS_CODEBUILD_CI
    _ = api.EnvEnum
    _ = api.Env
    _ = api.Config
    _ = api.Server
    _ = api.get_server
    _ = api.Ec2ConfigLoader
    _ = api.ConfigLoader
    _ = api.bsm


if __name__ == "__main__":
    from acore_server_config.tests import run_cov_test

    run_cov_test(__file__, "acore_server_config.api", preview=False)



# -*- coding: utf-8 -*-

"""
该模块用于在 EC2 实例中被调用, 并 "自省" 发现自己的服务器 ID 并在 AWS Parameter Store
找到对应的配置数据并读取.

注: 该模块只应在 EC2 内被运行, 它需要调用 AWS EC2 metadata API 获得自己的 instance id
从而继续发现自己的服务器 ID, 然后到 AWS Parameter Store 读取配置数据.

Usage:

.. code-block:: python

    >>> from acore_server_config.api import get_server
    >>> server = get_server()
    >>> server
    Server(id='sbx-blue', db_admin_password='sbx*dummy4test', db_username='myuser', db_password='sbx*dummy4test')
"""


import typing as T
import warnings

from s3pathlib import S3Path
from simple_aws_ec2.api import Ec2Instance
from acore_constants.api import TagKey

from .boto_ses import bsm as default_bsm
from .config.define import EnvEnum, Env, Config, Server

if T.TYPE_CHECKING:
    from boto_session_manager import BotoSesManager


def _get_default_s3folder_config(bsm: "BotoSesManager") -> str:
    return (
        S3Path(f"s3://{bsm.aws_account_id}-{bsm.aws_region}-artifacts")
        .joinpath(
            "projects",
            "acore_server_config",
            "config",
        )
        .to_dir()
    ).uri


def _get_default_parameter_name_prefix() -> str:
    return "acore_server_config"


def get_server(
    bsm: "BotoSesManager" = default_bsm,
    parameter_name_prefix: T.Optional[str] = None,
    use_s3: bool = True,
    use_parameter_store: bool = False,
    s3folder_config: T.Optional[str] = None,
    server_id: T.Optional[str] = None,
) -> Server:
    """
    在 EC2 上通过 "自省", 获得属于这个服务器的配置数据.

    配置数据的详细数据结构请参考 :class:`acore_server_config.config.define.server.Server`.

    .. note::
    
        从 0.3.1 开始, 该函数被 Ec2ConfigLoader 和 ConfigLoader 所替代, 该函数仍然保留
        但不建议使用.

    :param bsm: BotoSesManager 实例. 默认使用 EC2 上 IAM Role 所对应的.
    :param parameter_name_prefix: the parameter name prefix, the full name will
        be ${parameter_name_prefix}-${env_name}.
    :param use_s3: 是否从 S3 读取配置数据, 默认使用 S3, 因为配置数据可能会很大.
    :param use_parameter_store: 是否从 AWS Parameter Store 读取配置数据
    :param s3folder_config: S3 配置数据的根目录, 默认为
        s3://aws_account_id}-{aws_region}-artifacts/projects/acore_server_config/config/
    :param server_id: 强制指定 server_id, 跳过 "自省" 阶段. 常用于测试. 这个 server_id
        的格式为: ${env_name}-${server_name}, 例如: sbx-blue
    """
    warnings.warn(
        "you should not use this function anymore, use "
        "'acore_server_config.api.Ec2ConfigLoader' instead."
    )
    if sum([use_s3, use_parameter_store]) != 1:
        raise ValueError(
            "Only one of use_s3 and use_parameter_store can be True at the same time."
        )

    if server_id is None:
        ec2_inst = Ec2Instance.from_ec2_inside(bsm.ec2_client)
        server_id = ec2_inst.tags[TagKey.SERVER_ID]
    env_name, server_name = server_id.split("-", 1)
    if parameter_name_prefix is None:
        parameter_name_prefix = _get_default_parameter_name_prefix()
    parameter_name = f"{parameter_name_prefix}-{env_name}"

    if use_s3:
        if s3folder_config is None:
            s3folder_config = _get_default_s3folder_config(bsm=bsm)
        config = Config.read(
            env_class=Env,
            env_enum_class=EnvEnum,
            bsm=bsm,
            parameter_name=parameter_name,
            s3folder_config=s3folder_config,
        )
    elif use_parameter_store:
        config = Config.read(
            env_class=Env,
            env_enum_class=EnvEnum,
            bsm=bsm,
            parameter_name=parameter_name,
            parameter_with_encryption=True,
        )
    else:  # pragma: no cover
        raise NotImplementedError

    env = config.get_env(env_name=env_name)
    server = env.servers[server_name]
    return server
